package {
	import flash.text.TextField;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.display.MovieClip;
	import flash.events.MouseEvent;

	import com.baconandgames.math.geom.EllipseExt;

	/**
	 * @author Sean James McKenzie - http://www.baconandgames.com
	*/
	 
	public class EllipseDemoMain extends MovieClip {
		
		// movieClip to draw an ellipse in (just a visual representation of the Ellipse for the viewer)
		private var ellipseClip:MovieClip;
		// instance of EllipseExt we're going to use for this example
		private var el:EllipseExt;
		// draggable nodes
		private var nodeA:MovieClip;
		private var nodeB:MovieClip;
		// clip to draw line on
		private var lineClip:MovieClip;
		// points of intersection
		private var pi0:MovieClip;
		private var pi1:MovieClip;
		// extend line indefinitely through both nodes
		private var extend:Boolean;
		private var extendBtn:MovieClip;
		// testfield to display number of intersection points
		private var numIntersections:TextField;

		private static const ELLIPSE_CENTER:Point = new Point(250,140);
		private static const ELLIPSE_WIDTH:Number = 250;
		private static const ELLIPSE_HEIGHT:Number = 100;
		private static const STAGE_WIDTH:Number = 500;
		private static const STAGE_HEIGHT:Number = 280;
		
		public function EllipseDemoMain() {

			el = new EllipseExt(ELLIPSE_CENTER.x,ELLIPSE_CENTER.y,ELLIPSE_WIDTH,ELLIPSE_HEIGHT);
			el.center = ELLIPSE_CENTER;
			
			// draw an ellipse, just so we can see it
			ellipseClip = new MovieClip();
			addChild(ellipseClip);
			ellipseClip.graphics.lineStyle(1,0x000000,.2);
			ellipseClip.graphics.drawEllipse(0,0,ELLIPSE_WIDTH,ELLIPSE_HEIGHT);
			ellipseClip.x = (STAGE_WIDTH-ELLIPSE_WIDTH)/2;
			ellipseClip.y = (STAGE_HEIGHT-ELLIPSE_HEIGHT)/2;
						
			// create a movieClip to draw our line in
			lineClip = new MovieClip();
			addChild(lineClip);

			// create the draggable nodes	
			nodeA = drawCircle(new Point(50,50),5,0x443B31,true);
			nodeB = drawCircle(new Point(450,230),5,0x443B31,true);
			
			// create circles to mark points of intersection
			pi0 = drawCircle(new Point(0,0),2,0x443B31);
			pi1 = drawCircle(new Point(0,0),2,0x443B31);			
			
			// toggle extend button
			extendBtn = MovieClip(getChildByName("extend_btn"));
			extendBtn.addEventListener(MouseEvent.CLICK,extendBtnHandler,false,0,true);
			extendBtn.useHandCursor = true;
			extendBtn.buttonMode = true;
			extend = false;
			
			numIntersections = TextField(getChildByName("pts_txt"));
			
			// let's get to updatin' this biatch
			addEventListener(Event.ENTER_FRAME,update,false,0,true);
		}
		
		private function extendBtnHandler(event:MouseEvent):void {
			extend = (extend) ? false : true;
			if(extend){
				extendBtn.gotoAndStop(2);
			}else{
				extendBtn.gotoAndStop(1);
			}
		}

		private function update(e:Event):void{
			drawLine();
			// get points of intersection
			var pts:Array = el.getIntersections(new Point(nodeA.x,nodeA.y),new Point(nodeB.x,nodeB.y),extend);
			// show/hide intersection points
			var numPoints:int = 0;
			if(pts[0] != null){	
				pi0.x = pts[0].x;
				pi0.y = pts[0].y;
				pi0.visible = true;
				numPoints++;
			}else{
				pi0.visible = false;	
			}
			if(pts[1] != null){	
				pi1.x = pts[1].x;
				pi1.y = pts[1].y;
				pi1.visible = true;
				numPoints++;
			}else{
				pi1.visible = false;	
			}		
			numIntersections.text = "number of intersections: "+numPoints;		
		}
		
		// draws a circle (for demo functionality only)
		public function drawCircle(pt:Point,rad:Number,color:Number,draggable:Boolean=false):MovieClip{
			var circle:MovieClip = new MovieClip();
			circle.graphics.beginFill(color);
			circle.graphics.drawCircle(0,0,rad);
			circle.graphics.endFill();
			circle.x = pt.x;
			circle.y = pt.y;
			addChild(circle);
			// make node draggable
			if(draggable){
				circle.addEventListener(MouseEvent.MOUSE_DOWN,initDrag,false,0,true);
				circle.addEventListener(MouseEvent.MOUSE_UP,killDrag,false,0,true);
				circle.useHandCursor = true;
				circle.buttonMode = true;
			}			
			return circle;
		}
		
		// yes I know, this is a lame way to handle the dragging, but it's fine for these purposes
		private function initDrag(e:MouseEvent):void{e.currentTarget.startDrag();}
		private function killDrag(e:MouseEvent):void{e.currentTarget.stopDrag();}
		
		// draw the line between nodes A and B
		private function drawLine():void{
			lineClip.graphics.clear();
			lineClip.graphics.lineStyle(1,0x000000,.2);
			
			if(!extend){
				lineClip.graphics.moveTo(nodeA.x,nodeA.y);
				lineClip.graphics.lineTo(nodeB.x,nodeB.y);
			}else{
				var m:Number = (nodeB.y-nodeA.y)/(nodeB.x-nodeA.x);
				var si:Number = nodeB.y-(m*nodeB.x);
				var p0:Point = new Point(0,si);
				var p1:Point = new Point(STAGE_WIDTH,m*STAGE_WIDTH+si);
				lineClip.graphics.moveTo(p0.x,p0.y);
				lineClip.graphics.lineTo(p1.x,p1.y);
			}
		}
		
	}
}