package com.baconandgames.math.geom {
	import flash.geom.Point;

	import org.casalib.math.geom.Ellipse;

	/**
	 * @author Sean James McKenzie - http://www.baconandgames.com
	 */
	public class EllipseExt extends Ellipse {
		
		// For ellipse information see: http://www.analyzemath.com/EllipseEq/EllipseEq.html
		
		private var k:Number;	// center x
		private var h:Number;	// center y
		private var a:Number;	// radius x
		private var b:Number;	// radius y
		
		public var test:Number;
		
		public function EllipseExt(x:Number, y:Number, width:Number, height:Number) {
			super(x,y,width,height);
			h = x;
			k = y;
			a = width/2;
			b = height/2;
		}
		
		// returns array of intersection points, will return null if there is no intersection
		// will return 2 points (usually) in the array, or one if the line is a tangent
		// set extend to true if you want the line to extend indefinitely, false if only between the 2 points
		// setting sort to true will will put the intersection point closer to p1 in the first index of the array, handy
		// if you're doing line of sight between an enemy (p1) and a target (p2) and need to know which intersection point would
		// be the enemy's collision point...
		public function getIntersections(p1:Point,p2:Point,extend:Boolean=false,sort:Boolean=false):Array{
			var ar:Array = new Array();
			 // normailze points (ie, make everything relative to (0,0))
			var p1Norm:Point = new Point(p1.x-center.x,-(p1.y-center.y));
			var p2Norm:Point = new Point(p2.x-center.x,-(p2.y-center.y));			
			// get the slope - y = mx+b
			var m:Number = (p2Norm.y-p1Norm.y)/(p2Norm.x-p1Norm.x);
			// get the slope intercept
			var si:Number = p2Norm.y-(m*p2Norm.x);
			// get the coefficients
			var A:Number = (b*b)+(a*a*m*m);
			var B:Number = 2*a*a*si*m;
			var C:Number = a*a*si*si-a*a*b*b; 
			// vars to hold the new points
			var p3:Point = new Point();
			var p4:Point = new Point();
			// we're going to use the quadratic equation to find x, so let's see what's going to be under the radicand first
			// depending on this value, we can potentially skip a few calculations
			var radicand:Number = (B*B)-(4*A*C);
			if(radicand >= 0){
				// solve for x values - using the quadratic equation
				p3.x = (-B-Math.sqrt(radicand))/(2*A);
				p4.x = (-B+Math.sqrt(radicand))/(2*A);
				// calculate y, since we know it's on the line at that point (otherwise there would be no intersection)
				p3.y = m*p3.x+si;
				p4.y = m*p4.x+si;				
				// revert to flash coordinate system
				p3.x += center.x;
				p3.y = -p3.y+center.y;
				p4.x += center.x;
				p4.y = -p4.y+center.y;
				// add to array of points	
				if(!extend){	
					// only return points of the intersection that exist on the line between p1 and p2
					if (isBetweenPoints(p1,p2,p3)) ar.push(p3);
					if (isBetweenPoints(p1,p2,p4)) ar.push(p4);
				}else{
					// return points of intersection on the infinitly extending line defined by p1 and p2
					ar.push(p3);
					ar.push(p4);
				}				
				if(sort && ar.length > 1){
				// make sure that index 0 contains the closer point
					if(distanceBetweenPoints(p1,ar[0]) > distanceBetweenPoints(p1,ar[1])){
						ar.reverse();
					}
				}
			}else if(radicand == 0){
				// in this case both points will result in the same, so we only need to calculate one point
				p3.x = (-B-Math.sqrt(radicand))/(2*A);	
				p3.y = m*p3.x+si;				
				// revert to flash coordinate system
				p3.x += center.x;
				p3.y = -p3.y+center.y;
				// add to array of points				
				if(!extend){		
					if (isBetweenPoints(p1,p2,p3)) ar.push(p3);
				}
			}
			// cleanup the array
			if(ar.length == 0){
				ar = [null,null];	
			}else if(ar.length == 1){
				ar[1] = null;
			}
			return ar;
		}
		
		// shortcut method, returns true if line defined by p1 and p2 intesections the ellipse
		// use this if you just need to know if an intersection has occurred but don't need the points of interseciont (line of sight) 
		// this will save you a bit of overhead
		public function checkForIntersection(p1:Point,p2:Point):Boolean{
			// normailze points (ie, make everything relative to (0,0))
			var p1Norm:Point = new Point(p1.x-center.x,-(p1.y-center.y));
			var p2Norm:Point = new Point(p2.x-center.x,-(p2.y-center.y));			
			// get the slope - y = mx+b
			var m:Number = (p2Norm.y-p1Norm.y)/(p2Norm.x-p1Norm.x);
			// get the slope intercept
			var si:Number = p2Norm.y-(m*p2Norm.x);			
			// get the coefficients
			var A:Number = (b*b)+(a*a*m*m);
			var B:Number = 2*a*a*si*m;
			var C:Number = a*a*si*si-a*a*b*b;
			// the value below is what would be under the radicand in the quadratic equation, if less than zero ther are no intersec
			return ((B*B)-(4*A*C) < 0) ? false : true;
		}
		
		// shortcut method, returns the point of intersection between pt and the center of the ellipse
		public function getPointCenterIntersection(pt:Point):Point{
			// normalize point
			var ptNorm:Point = new Point(pt.x -= center.x,pt.y = -(pt.y-center.y));
			// solve for x and y
			var p:Point = new Point();
			p.x = ((a*b)/Math.sqrt(a*a*ptNorm.y*ptNorm.y+b*b*ptNorm.x*ptNorm.x))*ptNorm.x;
			p.y = ((a*b)/Math.sqrt(a*a*ptNorm.y*ptNorm.y+b*b*ptNorm.x*ptNorm.x))*ptNorm.y;
			// revert to original positioning
			p.x += center.x;
			p.y = -p.y+center.y; // flip y for flash coordinate system				
			return p;
		}
		
		// shortcut method, pass in a y value and it returns the intersection points of intersection with the ellipse at that value
		public function getInersectionsFromY(y:Number):Array{
			var ptA:Point = new Point();
			var ptB:Point = new Point();
			ptA.y = y;
			ptB.y = y;
			// normalize point				
			y = -(y-center.y);
			var x:Number = Math.sqrt(a*a-(a*a*y*y)/(b*b));
			ptA.x = x+center.x;
			ptB.x = -x+center.x;
			return [ptA,ptB];
		}
		
		// shortcut method, pass in an x value and it returns the intersection points of intersection with the ellipse at that value
		public function getInersectionsFromX(x:Number):Array{
			var ptA:Point = new Point();
			var ptB:Point = new Point();
			ptA.x = x;
			ptB.x = x;
			// normalize point				
			x = x-center.x;
			var y:Number = Math.sqrt(b*b-(b*b*x*x)/(a*a));
			ptA.y = y+center.y;
			ptB.y = -y+center.y;
			return [ptA,ptB];
		}	
		
		// returns true if pt is between anchor1 and anchor2
		private function isBetweenPoints(anchor1:Point,anchor2:Point,pt:Point):Boolean{
			var xMin:Number;
			var xMax:Number;
			var yMin:Number;
			var yMax:Number;
			// determine the x bounds
			if(anchor1.x < anchor2.x){
				xMin = anchor1.x;
				xMax = anchor2.x;
			}else{
				xMin = anchor2.x;
				xMax = anchor1.x;
			}
			// determine the y bounds
			if(anchor1.y < anchor2.y){
				yMin = anchor1.y;
				yMax = anchor2.y;
			}else{
				yMin = anchor2.y;
				yMax = anchor1.y;
			}
			// if between those values, point is between the anchors
			return (pt.x <= xMax && pt.x >= xMin && pt.y <= yMax && pt.y >= yMin) ? true : false;
		}
		
		private function distanceBetweenPoints(p1:Point,p2:Point):Number{
			var dx:Number = p2.x - p1.x, dy:Number = p2.y - p1.y;
			return Math.pow(dx * dx + dy * dy, .5);
		}
		
		
		
	}
}
